; Reverse-assembly of the file "AMAZMAZE.BIN"
; VideoCade #5001 Strategy series, circa 1979
;
; CPU Type: Z80 for Bally Professional Arcade
;
; Created by Richard C. Degler with dZ80 2.0
;    using script "BallyHVG.LUA" from scratch
;
; on Friday, 05 of November 2010 at 12:46 AM
;
; This code assembles with ZMac assembler
; command line: zmac -i -m AMAZMAZE.ASM
;

        NOLIST

        INCLUDE "HVGLIB.H"      ; HOME VIDEO GAME LIBRARY

; Amazing Maze memory defines
; 28 x 21 bytes [maximum] Walls build area
L4CA8   EQU     $4CA8           ; first Block for Hard start
L4D56   EQU     $4D56           ; first Block for Medium start
L4DD7   EQU     $4DD7           ; first Block for Easy start
; ... all through $4EF4           ; last Block for ALL build areas
; 62 bytes of System Stack area
; note: first of 19 Variables cleared [also Top of Amazing Maze Stack]
L4F35   EQU     $4F35           ; byte Player 2 Door found already
L4F36   EQU     $4F36           ; byte Build Elsewhere Wrap enable
L4F37   EQU     $4F37           ; byte automatic Player 2 Delay up-counter
L4F38   EQU     $4F38           ; byte Player 1 Door Y position
L4F39   EQU     $4F39           ; byte Player 2 Door Y position
L4F3A   EQU     $4F3A           ; byte Player 1 old X location
; L4F3B   EQU     $4F3B           ; byte Player 1 old Y [not directly used]
L4F3C   EQU     $4F3C           ; byte Player 1 new X location
; L4F3D   EQU     $4F3D           ; byte Player 1 new Y [not directly used]
L4F3E   EQU     $4F3E           ; byte Player 1 Delta X direction
L4F3F   EQU     $4F3F           ; byte Player 1 Delta Y direction
L4F40   EQU     $4F40           ; byte Player 2 old X location
; L4F41   EQU     $4F41           ; byte Player 2 old Y [not directly used]
L4F42   EQU     $4F42           ; word Head of Correct trail
L4F44   EQU     $4F44           ; byte Player 2 new X location
; L4F45   EQU     $4F45           ; byte Player 2 new Y [not directly used]
L4F46   EQU     $4F46           ; byte Player 2 Delta X direction
L4F47   EQU     $4F47           ; byte Player 2 Delta Y direction
;
; Player Scores [00 to 99] displayed by ReSTart #2 [for both AM and TTT]
L4F48   EQU     $4F48           ; 2 BCD digits Player 1 Score
L4F49   EQU     $4F49           ; 2 BCD digits Player 2 Score
;
; 5 Default parameters [by Difficulty]
L4F4A   EQU     $4F4A           ; byte Maze Height
L4F4B   EQU     $4F4B           ; byte Maze Width
L4F4C   EQU     $4F4C           ; byte X & Y Quantum
L4F4D   EQU     $4F4D           ; byte Playfield Width
L4F4E   EQU     $4F4E           ; byte Playfield Height
L4F4F   EQU     $4F4F           ; byte Difficulty [1=Easy through 3=Hard]
;
; Winning Path tracking
L4F50   EQU     $4F50           ; store byte L4F53 during play [0/1 nibble of...]
L4F51   EQU     $4F51           ; store word L4F54 during play [direction list head]
L4F53   EQU     $4F53           ; byte 0/1 = Current path Nibble tracker of...
L4F54   EQU     $4F54           ; word Current Direction path List element pointer
; L4F56 through L4FB9             ; (up to) 100 bytes of Winning Path direction nibbles
L4FBA   EQU     $4FBA           ; init to 4 for final Direction, put as Head into L4F54
; 10 unused bytes
L4FC5   EQU     $4FC5           ; In-Game [cleared by RST #4], also Top of MUSIC Stack area
;

; Tic-Tac-Toe memory defines
; L4F48   [see above]             ; 2 BCD digits Player 1 Score [00 to 99]
; L4F49   [see above]             ; 2 BCD digits Player 2 Score [00 to 99]
; 56 bytes of System Stack area
; note: first of 19 Variables cleared [also Top of Tic-Tac-Toe Stack]
L4FA0   EQU     $4FA0           ; Joystick tracking, also [un-used] Square #0 location
L4FA1   EQU     $4FA1           ; 9 bytes [thru L4FA9] (4='X', 1='O') in Square #1 to 9
L4FAA   EQU     $4FAA           ; 8 bytes [thru L4FB1] Line added up Results [0 to 7]
L4FB2   EQU     $4FB2           ; byte Squares Filled [0 thru 8 (done if 9)]
L4FB3   EQU     $4FB3           ; byte Who's Turn [0/1] [read by ReSTart #6]
L4FB4   EQU     $4FB4           ; byte Went First [0/1] [not used, just toggled]
L4FB5   EQU     $4FB5           ; byte Player 1 Square tracking [1 to 9]
L4FB6   EQU     $4FB6           ; byte Player 2 Square tracking [1 to 9]
; 14 unused bytes
; L4FC5   [see above]             ; [cleared by RST #4 - but not used by TTT ??]
;

        LIST

        ORG     $2000

; ********************* Start of Z80 Source Code
        DB      'U'             ; sentinal for Menu Start

; ========= First Menu Item node ===
        DW      L201C           ; ... Next Menu Node
        DW      L2505           ; ... Text "AMAZING MAZE"
        DW      L2022           ; ... Start Amazing Maze

; +++++++++ user ReSTart vectors 1 to 6 +++
        JP      L2309           ; ReSTart #1 (RST $08) to find Walls location [AM only]

        JP      L2438           ; ReSTart #2 (RST $10) to display Scores [for both]

        JP      L25CB           ; ReSTart #3 (RST $18) to (un-)draw Both Boxes [AM only]

        JP      L226B           ; ReSTart #4 (RST $20) to clear In-Game flag [for both]

        JP      L267E           ; ReSTart #5 (RST $28) to (un-)draw Selector Box [TTT only]

        JP      L2270           ; ReSTart #6 (RST $30) to get Who's Turn [TTT only]

        NOP                     ; or DB  $00 filler byte

; ++++++ screen INTerrupt VECtor [used by AM only] +++
; L201A:
        DW      L2295

; ====== Second Menu Item node ===
L201C:  DW      MENUST          ; ... System Menu Start Node
        DW      L2460           ; ... Text "TIC-TAC-TOE"
        DW      L25ED           ; ... Start Tic Tac Toe

; ********* Amazing Maze code starts here ***
L2022:  CALL    L21D0           ; do Set-up Number of Players and Max Score
        LD      HL,L202A        ; point to Difficulty 1, 2, or 3 sub-menus
        SYSTEM  MENU            ; UPI display a MENU
        ; [and branch to selection]

; === Difficulty sub-node #1 ===
L202A:  DW      L2030           ; ... Next Sub-Node
        DW      L24E3           ; ... Text "EASY"
        DW      L203C           ; ... Set to Easy

; === Difficulty sub-node #2 ===
L2030:  DW      L2036           ; ... Next Sub-Node
        DW      L24E8           ; ... Text "MEDIUM"
        DW      L203F           ; ... Set to Medium

; === Difficulty sub-node #3 ===
L2036:  DW      $0000           ; ... Final Sub-Node
        DW      L24EF           ; ... Text "HARD"
        DW      L2042           ; ... Set to Hard

; *** set Difficulty to Easy [on select 1,]
L203C:  LD      A,$01
        DB      $21             ; [then LD  HL,$023E and LD  HL,$033E]

; *** [or] set to Medium [on select 2,]
L203F:  LD      A,$02
        DB      $21             ; [then LD  HL,$033E]

; *** [or] set to Hard [on select 3]
L2042:  LD      A,$03
; [... and continue with A = Difficulty] ***
        DI                      ; +++ Disable Interrupts
        LD      SP,L4F35        ; set Stack Pointer at lowest ScratchPad location
        PUSH    AF
        CALL    L21B3           ; do Set-up Interrupt Vector and Clear Screen
        POP     AF
        LD      (L4F4F),A       ; write to Difficulty, then look-up Table...
        SYSSUK  INDEXW          ; UPI INDEX Word by A [1, 2 or 3]
        DW      L2449           ; ... Base Address = 9289
        ;
        EX      DE,HL           ; point to Default table by Difficulty
        LD      BC,$0005
        LD      DE,L4F4A        ; copy five Default settings
        LDIR
L205D:  SYSTEM  INTPC           ; UPI INTerPret with Context create
        ;
        DO      RECTAN          ; UPI paint a RECTANgle - for background
        DB      $05             ; ... X = 5
        DB      $02             ; ... Y = 2
        DB      $98             ; ... Width = 152
        DB      $4F             ; ... Height = 79
        DB      $FF             ; ... Color = Green
        ;
        DO      RECTAN          ; UPI paint a RECTANgle - for left turret
        DB      $04             ; ... X = 4
        DB      $00             ; ... Y = 0
        DB      $1A             ; ... Width = 26
        DB      $02             ; ... Height = 2
        DB      $F0             ; ... Color = Green and Yellow stripe [G G Y Y]
        ;
        DO      RECTAN          ; UPI paint a RECTANgle - for right turret
        DB      $84             ; ... X = 132
        DB      $00             ; ... Y = 0
        DB      $1A             ; ... Width = 26
        DB      $02             ; ... Height = 2
        DB      $F0             ; ... Color = Green and Yellow stripe
        ;
        DO      RECTAN          ; UPI paint a RECTANgle - for top of wall
        DB      $1C             ; ... X = 28
        DB      $02             ; ... Y = 2
        DB      $6A             ; ... Width = 106
        DB      $02             ; ... Height = 2
        DB      $F0             ; ... Color = Green and Yellow stripe
        ;
        DO      RECTAN          ; UPI paint a RECTANgle - for top of P1 castle
        DB      $08             ; ... X = 8
        DB      $0A             ; ... Y = 10
        DB      $0A             ; ... Width = 10
        DB      $02             ; ... Height = 2
        DB      $AF             ; ... Color = Red and Green stripe [R R G G]
        ;
        DO      RECTAN          ; UPI paint a RECTANgle - for top of P2 castle
        DB      $90             ; ... X = 144
        DB      $0A             ; ... Y = 10
        DB      $0A             ; ... Width = 10
        DB      $02             ; ... Height = 2
        DB      $5F             ; ... Color = Blue and Green stripe [B B G G]
        ;
        DO      RECTAN          ; UPI paint a RECTANgle - for rest of P1 castle
        DB      $09             ; ... X = 9
        DB      $0C             ; ... Y = 12
        DB      $08             ; ... Width = 8
        DB      $09             ; ... Height = 9
        DB      $AA             ; ... Color = Red
        ;
        DO      RECTAN          ; UPI paint a RECTANgle - for rest of P2 castle
        DB      $91             ; ... X = 145
        DB      $0C             ; ... Y = 12
        DB      $08             ; ... Width = 8
        DB      $09             ; ... Height = 9
        DB      $55             ; ... Color = Blue
        ;
        DONT    XINTC           ; UPI eXit INTerpreter with Context
        ;
        RST     $10             ; User ReStarT #2 to Display Scores
        CALL    L2400           ; do Display "GET READY" string
        LD      SP,L4F35        ; re-set Stack Pointer at lowest ScratchPad location
        RST     $20             ; User ReStarT #4 to Clear In-Game flag
        CALL    L2199           ; do Silence all sounds
        EI                      ; +++ Enable Interrupts
; *** initialize Maze building
L209C:  LD      A,$BE           ; output VERtical BLanking line 95 * 2 = 190
        OUT     (VERBL),A
        SYSSUK  FILL            ; UPI FILL memory with data
        DW      L4CA8           ; ... Memory Address = 19624 [line 81]
        DW      $024D           ; ... Byte Count = 589 [to $4EF4]
        DB      $00             ; ... Data = 0 to clear Walls area
        ;
        SYSSUK  FILL            ; UPI FILL memory with data
        DW      L4F35           ; ... Memory Address = 20277
        DW      $0013           ; ... Byte Count = 19 [to L4F47]
        DB      $00             ; ... Data = 0 to clear Player variables
        ;
        SYSSUK  FILL            ; UPI FILL memory with data
        DW      L4F53           ; ... Memory Address = 20307
        DW      $0067           ; ... Byte Count = 103 [to L4FB9]
        DB      $00             ; ... Data = 0 to clear all Path nibbles
        ;
        LD      HL,L4FBA        ; point at Direction path List tail
        LD      (HL),$04        ; [last Direction will be a "Left"]
        LD      (L4F54),HL      ; store Direction path List element
        LD      A,(L4F4A)       ; read Maze Height...
        LD      E,A             ; and save it
        SYSTEM  RANGED          ; UPI RANGED random number
        ;
        LD      (L4F38),A       ; write Player 1 Door Y position
        LD      B,A
        LD      C,$00
        RST     $08             ; User ReStarT #1 to find Walls location
        LD      (HL),$01        ; mark it with 1 for "Visited"
; *** build Maze until done
L20CC:  LD      A,L             ; output HL & $00FF to VIBRAto
        OUT     (VIBRA),A
        CALL    L23A8           ; do Check Moves Available sub-routine
        LD      A,E             ; [number of Directions found]
        OR      A               ; set flags and...
        JR      Z,L20DB                 ; (+$05) break if No move
        CALL    L2334           ; do Pick a Direction sub-routine
        JR      L20CC                   ; (-$0f) loop to Build Maze

; *** more build Maze if No directions availiable ***
L20DB:  LD      A,(L4F35)       ; read Player 2 Door Found yet
        OR      A               ; set flags and...
        JR      Z,L209C                 ; (-$45) to Reinitialize if Not found
; [...else far enough along to Keep Path and start elsewhere]
        CALL    L22E8           ; do Build maze Elsewhere sub-routine
        JR      C,L20CC                 ; (-$1a) loop to Build Maze, until ...
; *** Maze built, now show it
        CALL    L2400           ; do Un-display "GET READY" string [all Green]
        LD      A,(L4F4F)       ; read from Difficulty
        SYSSUK  INDEXW          ; UPI INDEX Word by A
        DW      L232C           ; ... Base Address = 9004
        ;
        EX      DE,HL           ; get Walls Build Area start
        DEC     HL              ; bump down
        LD      DE,$0518        ; initialize X for 24, Y for 5 [upper left]
; *** display Wall Blocks according to Build area
L20F5:  INC     HL              ; next Block information
        PUSH    HL
        PUSH    DE
        LD      A,(HL)          ; read Wall Block shape
        OR      A               ; set flags, and if none ...
        JR      Z,L213B                 ; (+$3f) exit to End Music, etc...
        RRA                     ; align Shape [down past Visited flag]
        AND     00000011B       ; mask with $03
        LD      E,A
        LD      A,(L4F4F)       ; read from Difficulty
        SYSSUK  INDEXB          ; UPI INDEX Byte by A (1 to 3)
        DW      L246F           ; ... Base Address = 9327
        ;
        ADD     A,E             ; 8, 4, or 0 plus 0 to 3 for Wall shape
        SYSSUK  INDEXW          ; UPI INDEX Word by A (0 to 11)
        DW      L2473           ; ... Base Address = 9331
        ;
        EX      DE,HL           ; get Block pattern ...
        LD      A,$28           ; for X-OR and EXPAND magic
        POP     DE              ; ... to stamp in place
        PUSH    DE
        SYSTEM  WRITP           ; UPI WRITe with Pattern size lookup
        ;
; *** compute Next Block location
        POP     DE
        LD      BC,(L4F4C)      ; read X & Y Quantum and Playfield Width
        LD      A,C
        ADD     A,E             ; add quantum to X
        LD      E,A
        CP      B
        JR      NZ,L2138                ; (+$1a) next if X not Right edge yet
        LD      E,$18           ; X back to Left edge
        LD      A,C
        ADD     A,D             ; add quantum to Y
        LD      D,A
        LD      A,(L4F4E)       ; read Playfield Height
        CP      D
        JR      NZ,L2138                ; (+$0f) next row if Y not through
; *** else done, so cover top of Walls Build area...
        PUSH    HL
        PUSH    DE
        SYSSUK  FILL            ; UPI FILL memory with data (for lines 81 to 92)
        DW      L4CA8           ; ... Memory Address = 19624
        DW      $01E0           ; ... Byte Count = 480 (12 scanlines)
        DB      $FF             ; ... Data = Green [only up to $4E87]
        ;
        LD      A,$BA           ; output VERtical BLanking line 93 * 2 = 186
        OUT     (VERBL),A       ; to hide bottom of Walls Build area
        POP     DE
        POP     HL
L2138:  POP     HL
        JR      L20F5                   ; (-$46) loop back for Next Block

; *** Exit here when Maze drawn ***
L213B:  SYSTEM  EMUSIC          ; UPI End playing MUSIC
        ;
        LD      E,$15           ; for left side
        LD      A,(L4F38)       ; read Player 1 Door Y position
        CALL    L2281           ; do Erase a 3 x 3 Box
        LD      (L4F3A),HL      ; write Player 1 old XY locations
        LD      DE,(L4F4D)      ; read Playfield Width and Height
        LD      A,(L4F39)       ; read Player 2 Door Y position
        CALL    L2281           ; do Erase a 3 x 3 Box
        LD      (L4F40),HL      ; write Player 2 old XY locations
        RST     $18             ; User ReStarT #3 to Pre-Draw Both Boxes
        CALL    L21A8           ; do Move P2 Left one step
        LD      (L4F42),HL      ; write as Head of Correct trail
        SYSSUK  PAWS            ; UPI PAUSE for 4.25 seconds
        DB      $FF             ; ... Interrupts = 255
        ;
        CALL    L2199           ; do Silence all sounds
        LD      A,(L4F53)       ; copy Current path Nibble [1/0] ...
        LD      (L4F50),A       ; to storage
        BIT     0,A
        LD      HL,(L4F54)      ; read Direction path List element
        JR      Z,L2170                 ; (+$01) skip if Nibble even
        INC     L               ; skip to next byte if Nibble was odd
L2170:  LD      (L4F54),HL      ; write as Direction path List element...
        LD      (L4F51),HL      ; and copy to storage
        LD      A,(NUMPLY)      ; read NUMber of PLaYers
        CP      $01
        JR      NZ,L2185                ; (+$08) skip if Two players
        RST     $18             ; User ReStarT #3 to Erase Both Boxes
        CALL    L21A8           ; do Move P2 Left one step [into maze]
        LD      (L4F40),HL      ; write Player 2 old XY locations
        RST     $18             ; User ReStarT #3 to Draw Both Boxes
L2185:  LD      HL,L4FC5        ; set In-Game flag before SENSE #1 loop
        LD      (HL),$01
; *** back here for SENSE transition #1 loop
L218A:  SYSSUK  SENTRY          ; UPI SENse TRansition
        DW      ALKEYS          ; ... Keymask Address for All KEYS
        ;
        SYSSUK  DOIT            ; UPI DOIT table #1, branch to translation handler
        DW      L24FB           ; ... Table Address for TWO Joysticks and Keypad
        ;
        CALL    L21F1           ; do Check for Win, otherwise...
        JR      L218A                   ; (-$0d) loop to SENSE1

; *** on sentry Key Down handler [for both AM and TTT] ***
L2197:  SYSTEM  PIZBRK          ; UPI take a PIZza BReaK
        ;
; ****** SILENCE all sounds routine ***
L2199:  PUSH    BC
        PUSH    HL
        SYSTEM  EMUSIC          ; UPI End playing MUSIC
        ;
        LD      BC,$0818        ; output eight Presets to Sound Block
        LD      HL,L24F3
        OTIR
        POP     HL
        POP     BC
        RET

; *** Move Player Two Left one step routine ***
L21A8:  LD      HL,(L4F40)      ; read Player 2 old XY locations
        LD      A,(L4F4C)       ; read X & Y Quantum
        NEG
        ADD     A,L
        LD      L,A
        RET

; ****** Set-up Interrupt Vector and Clear Screen ***
L21B3:  LD      A,$20
        LD      I,A             ; +++ set interrupt page, and...
        LD      A,$1A           ; output INterrupt FeedBacK for L201A
        OUT     (INFBK),A
        IM      2               ; +++ set standard Bally mode
        SYSSUK  FILL            ; UPI FILL memory with data
        DW      L4F48           ; ... Memory Address = 20296
        DW      $0002           ; ... Byte Count = 2
        DB      $00             ; ... Data = 0 to reset both scores
        ;
        SYSSUK  COLSET          ; UPI COLors SET
        DW      L246C           ; ... Table Address = 9324
        ;
; *** Clear Screen sub-routine ***
L21C8:  SYSSUK  FILL            ; UPI FILL memory with data
        DW      $4000           ; ... Memory Address = 16384 [NORMEM]
        DW      $0F00           ; ... Byte Count = 3840 [96 scanlines]
        DB      $00             ; ... Data = 0 to clear screen
        ;
        RET

; ****** Set-up Game Parameters routine [both AM and TTT] ***
L21D0:  SYSSUK  GETPAR          ; UPI GET game PARameter from user
        DW      NOPLAY          ; ... Prompt Address for "# of PLAYERS"
        DB      $01             ; ... BCD Digits = 1
        DW      NUMPLY          ; ... Parameter Address (1 or 2 only)
        ;
        LD      A,(HL)          ; store result
        OR      A
        JR      Z,L21D0                 ; (-$0b) try again if 0
        ADD     A,$FD           ; [already stored] plus 253
        JR      C,L21D0                 ; (-$0f) go again if > 2
        SYSSUK  GETPAR          ; UPI GET game PARameter from user
        DW      MXSCR           ; ... Prompt Address for "MAX SCORE"
        DB      $02             ; ... BCD Digits = 2
        DW      ENDSCR + 2      ; ... Parameter Address (last byte)
        ;
        RET

; ****** Play music score ***
L21E7:  PUSH    HL
        SYSSUK  BMUSIC          ; UPI Begin playing MUSIC
        DW      L4FC5           ; ... Music Stack = 20421
        DB      $0C             ; ... Voices = 12 (Tone C)
        DW      L2408           ; ... Score Address = 9224
        ;
        POP     HL
        RET

; *** Check for Win sub-routine (called once per SENSE #1 loop) ***
L21F1:  LD      A,(L4F3A)       ; read Player 1 old X location
        LD      B,A             ; output P1 Xpos to VIBRAto
        OUT     (VIBRA),A
        LD      A,(L4F4D)       ; read Playfield Width
        CP      B
        JR      Z,L2216                 ; (+$19) on Player One won
        LD      A,(L4F40)       ; read Player 2 old X location
        OUT     (TONMO),A       ; output P2 Xpos to TONe Master Oscillator
        CP      $15
        RET     NZ              ; Return if Player Two not won yet, else...
        RST     $20             ; User ReStarT #4 to Clear In-Game flag
        LD      DE,L24DE
        RST     $10             ; User ReStarT #2 to Display Scores
        LD      HL,L4F49        ; point at Player Two Score
; *** here on either player won
L220D:  CALL    L21E7           ; do Play Music score
        LD      A,(HL)          ; increment player BCD score
        INC     A
        DAA
        LD      (HL),A
        JR      L2220                   ; (+$0a) to Someone won

; *** Player One won stub ***
L2216:  RST     $20             ; User ReStarT #4 to Clear In-Game flag
        LD      DE,L24D9
        RST     $10             ; User ReStarT #2 to Display Scores
        LD      HL,L4F48        ; point at Player One Score
        JR      L220D                   ; (-$13) to Either player won

; *** Someone Won continued ***
L2220:  PUSH    HL
        RST     $10             ; User ReStarT #2 to Display Scores
        LD      A,(L4F50)       ; copy from storage...
        LD      (L4F53),A       ; write Current path Nibble [1/0]
        LD      HL,(L4F51)      ; copy from storage...
        LD      (L4F54),HL      ; write Current path List element
        LD      HL,(L4F42)      ; read Head of Correct trail
; *** loop here until Path drawn
L2231:  LD      A,$15
        CP      L               ; is marker at left edge ?
        JR      NC,L224A                ; (+$14) exit if there
        PUSH    HL
        PUSH    DE
        EX      DE,HL
        LD      A,$10           ; for OR color magic write [draw Marker]
        SYSTEM  WRITP           ; UPI WRITe with Pattern size lookup
        ;
        CALL    L2512           ; do Read current P2 Direction Nibble
        POP     DE
        POP     HL
        LD      A,(L4F4C)       ; read X & Y Quantum [to add or subtract]
        CALL    L2564           ; do Joystick handler on B [move Marker]
        JR      L2231                   ; (-$19) loop for more Path

; *** exit continued ***
L224A:  POP     HL
        CALL    L2251           ; do check for Last Game
        JP      L205D           ; re-run if it returns

; ****** check for Last Game [for Both] ***
L2251:  SYSSUK  PAWS            ; UPI PAUSE - for approximately 4.25 seconds
        DB      $FF             ; ... Interrupts = 255 [allows Music to finish]
        ;
        LD      A,(ENDSCR + 2)  ; read END SCoRe (last byte)
        CP      (HL)
        JR      NZ,L225C                ; (+$02) to SENSE2 transition loop
        SYSTEM  QUIT            ; UPI QUIT cartridge execution
        ; [Display "GAME OVER", then reset to menu]

; *** back here for SENSE transition #2 loop ***
L225C:  SYSSUK  SENTRY          ; UPI SENse TRansition
        DW      ALKEYS          ; ... Keymask Address for All KEYS
        ;
        SYSSUK  DOIT            ; UPI DOIT table #2, branch to translation handler
        DW      L2501           ; ... Table Address for NO Joysticks, Keys only
        ;
        IN      A,(SW0)         ; input Player One Joystick
        BIT     4,A             ; if Trigger One NOT pulled, then
        JR      Z,L225C                 ; (-$0e) loop to SENSE2
        RET                     ; else bail out...

; ++++++ RST $20 / ReSTart #4 to Clear In-Game flag [for both ??] +++
L226B:  XOR     A
        LD      (L4FC5),A
        RET

; ++++++ RST $30 / ReSTart #6 to get Which Player's turn [TTT only] +++
L2270:  LD      A,(L4FB3)       ; read Player Turn [0=#1, 1=#2 or Computer]
        OR      A
        RET

; ****** add Filled Squares in Line [misplaced] sub-routine [for Tic-Tac-Toe] ***
; with: C=Quantum, HL=first Square, and DE=Line Result [post-incremented]
L2275:  PUSH    BC
        LD      B,$00
        LD      A,(HL)          ; read Square #1
        ADD     HL,BC
        ADD     A,(HL)          ; plus Square #2
        ADD     HL,BC
        ADD     A,(HL)          ; plus Square #3
        LD      (DE),A          ; save Result in Line [0 to 12]
        INC     DE
        POP     BC
        RET                     ; with HL still pointed at last Square

; *** Erase a 3 x 3 BOX routine [with A=Door Y location, and E = X] ***
L2281:  LD      BC,(L4F4B)      ; read Maze Width and X & Y Quantum
        LD      C,A
        XOR     A
L2287:  ADD     A,C             ; Y equals A times (Quantum + 1)
        DJNZ    L2287                   ; (-$03)
        ADD     A,$05           ; Y = Y + 5
        LD      D,A
        XOR     A               ; for $00 to draw in Yellow
        LD      BC,$0303
        SYSTEM  RECTAN          ; UPI paint a RECTANgle
        ;
        EX      DE,HL
        RET

; +++++++++ Interrupt Routine [for AM only] +++
L2295:  EX      AF,AF'
        EXX
        PUSH    IX              ; store environment
        PUSH    IY
        LD      A,$A2           ; output INterrupt LINe 81 * 2 = 182
        OUT     (INLIN),A
        LD      A,(L4FC5)       ; read In-Game flag
        OR      A
        JR      Z,L22DD                 ; (+$38) done if not In-Game
        LD      HL,L4F37        ; point at P2 Delay up-counter
        INC     (HL)            ; and increment it
        BIT     0,(HL)
        PUSH    AF
        CALL    Z,L2579         ; do Interrupt Sub-routine for X if even
        POP     AF
        CALL    NZ,L258E        ; do Interrupt Sub-routine for Y if odd
        LD      A,(NUMPLY)      ; read NUMber of PLaYers
        CP      $02
        JR      Z,L22DD                 ; (+$23) done if P2 is Human
        LD      A,(L4F37)       ; read P2 Delay up-counter
        OR      A
        JR      NZ,L22C6                ; (+$06) skip unless 0, then Start move
        CALL    L2512           ; do Read current P2 Direction Nibble
        CALL    L253D           ; do Apply Joystick Deltas [to P2]
L22C6:  LD      HL,L4F37        ; point at P2 Delay up-counter
        LD      A,(L4F4C)       ; read X & Y Quantum [for distance travelled]
        ADD     A,A             ; times 2 [keep moving for 8, 10, or 12 ticks]
        SUB     (HL)            ; check difference
        JR      NZ,L22D6                ; (+$06) skip unless same
        LD      D,A             ; else Stop movement
        LD      E,D
        LD      (L4F46),DE      ; write zero to Player 2 Joystick deltas
L22D6:  LD      A,$10
        CP      (HL)            ; check again
        JR      NZ,L22DD                ; (+$02) skip unless 16
        LD      (HL),$FF        ; reset Delay counter to -1
L22DD:  CALL    STIMER          ; do L0200 on-board routine
        EX      AF,AF'          ; then exit interrupt routine
        EXX
        POP     IY              ; restore environment
        POP     IX
        EI                      ; +++ re-Enable Interrupts
        RET

; ****** Build more Maze Elsewhere sub-routine ***
; [note: blocked in, so find another Visited wall]
L22E8:  INC     C
        LD      A,(L4F4B)       ; read Maze Width
        SUB     C               ; at right edge ?
        JR      Z,L22F5                 ; (+$06) to Wrap
; *** back from Wrap [unless Wrap Around disabled]
L22EF:  RST     $08             ; User ReStarT #1 to find Walls location
        LD      A,(HL)
        RRCA
        JR      NC,L22E8                ; (-$0c) loop if Not visited
        RET                     ; Carry set means More to find

; *** Wrap to next row if at Right edge ***
L22F5:  LD      C,A             ; reset Current X to zero
        INC     B               ; for next Current Y
        LD      A,(L4F4A)       ; read Maze Height
        SUB     B               ; at Bottom edge ?
        JR      NZ,L22EF                ; (-$0e) if Not
; *** Wrap Around if at Bottom edge
        LD      B,A             ; reset Current Y to zero
        LD      A,(L4F36)       ; read Enable to Wrap Around
        OR      A               ; set flags [Carry cleared]
        RET     Z               ; done if Wrap Disabled
        XOR     A
        LD      (L4F36),A       ; write zero to Disable
        JR      L22EF                   ; (-$1a) continue at Upper Left

; ++++++ RST $08 / ReSTart #1 to Find Walls location (by BC) +++
; [with B=Y (example P1 Door), C=X (0=left edge)]
L2309:  PUSH    DE
        PUSH    AF
        LD      A,(L4F4B)       ; read Maze Width
        LD      E,A
        XOR     A
        LD      D,A             ; expand into DE
        LD      H,A
        LD      L,A             ; HL = $0000
        LD      A,B
        PUSH    BC
        LD      B,$08           ; Multiply Grid Y * Width
L2317:  ADD     HL,HL
        RLA                     ; bit test 7 thru 0
        JR      NC,L231C                ; (+$01)
        ADD     HL,DE
L231C:  DJNZ    L2317                   ; (-$07)
        POP     BC
        LD      E,C             ; read Grid X
        ADD     HL,DE
        PUSH    HL              ; store byte offset
        LD      A,(L4F4F)       ; read from Difficulty
        SYSSUK  INDEXW          ; UPI INDEX Word by A
        DW      L232C           ; ... Base Address = 9004
        ;
        POP     HL              ; restore byte offset
        ADD     HL,DE           ; for memory location
        POP     AF
; [also entry #0 walls area table (not used)]
L232C:  POP     DE
        RET

; ====== Walls Area Start table [all should end at $4EF4] ===
        DW      L4DD7           ; Entry #1 for Easy start
        DW      L4D56           ; Entry #2 for Medium start
        DW      L4CA8           ; Entry #3 for Hard start

; ****** Pick a Random direction Build Maze sub-routine ***
; [note: D=Directions (bits 0=Up, 1=Down, 2=Left, 3=Right), E=Count]
L2334:  LD      A,$01           ; enable Build Elsewhere Wrap
        LD      (L4F36),A
        LD      A,E             ; number of Directions found
        SYSTEM  RANGED          ; UPI RANGED random number
        ;
        LD      E,A             ; for pick Only one Direction
        LD      A,D
        INC     E
        DEC     E               ; set Zero flag [unaffected by RRCA]
        RRCA                    ; [Carry if Rotated out of bit 0]
        JR      NC,L2346                ; (+$03) skip if Not Up
        JR      Z,L2384                 ; (+$3f) to stub if Picked
        DEC     E               ; else count down Random
L2346:  RRCA
        JR      NC,L234C                ; (+$03) skip if Not Down
        JR      Z,L2389                 ; (+$3e) to stub if Picked
        DEC     E               ; else count down Random
L234C:  RRCA
        JR      NC,L2351                ; (+$02) skip if Not Left
        JR      Z,L23A3                 ; (+$52) to stub if Picked
L2351:  RST     $08             ; User ReStarT #1 to find Walls location
        LD      A,(HL)          ; Must be Right [in-line "stub"]
        OR      00000010B       ; mix in $02 "OK Right" Wall attribute here
        LD      (HL),A
        LD      A,(L4F4B)       ; read Maze Width - 1
        DEC     A
        CP      C
        JR      Z,L2393                 ; (+$36) if at Right edge already
        INC     C               ; move Right a Block
        LD      A,01000001B     ; $41 for Wall Attributes
; *** stubs return here [bits 0=Visited, 1=OK Right, 2=OK Down,3=0, and 4-7=Direction]
L2360:  RST     $08             ; User ReStarT #1 to find Walls location
        OR      (HL)
        LD      (HL),A          ; mix in Wall attributes
        LD      D,A             ; save for Direction value
        LD      A,(L4F35)       ; read Player 2 Door Found yet
        OR      A
        RET     NZ              ; bail if Winning Path already built
; note: Direction bits 0=Down, 1=Up, 2=Right, and 3=Left [why reversed ??]
        LD      HL,L4F53        ; toggle Current path Nibble tracker
        LD      A,(HL)
        XOR     $01
        LD      (HL),A
        LD      HL,(L4F54)      ; read Direction path List byte
        LD      A,D             ; un-save Direction value
        JR      NZ,L237B                ; (+$05) if now on high Nibble
        LD      (HL),A          ; write as high Nibble
        XOR     A
        RRD                     ; shift to low Nibble
        RET

L237B:  AND     $F0             ; mask for high Nibble
        OR      (HL)
        LD      (HL),A          ; save with low Nibble
        DEC     HL
        LD      (L4F54),HL      ; move to another Byte
        RET

; *** stub if Up Picked ***
L2384:  DEC     B               ; move Up a Block
        LD      A,00100101B     ; $25 for Wall Attributes
        JR      L2360                   ; (-$29)

; *** stub if Down Picked ***
L2389:  RST     $08             ; User ReStarT #1 to find Walls location
        LD      A,(HL)
        OR      00000100B       ; mix in $04 "OK Down" Wall attribute here
        LD      (HL),A
        INC     B               ; move Down a Block
        LD      A,00010001B     ; $11 for Wall Attributes
        JR      L2360                   ; (-$33)

; *** Done if Right at right edge ***
L2393:  LD      A,B
        LD      (L4F39),A       ; write Player 2 Door Y position
        LD      A,$01
        LD      (L4F35),A       ; write 1 for Player 2 Door Found
        LD      C,$00
        LD      A,(L4F38)       ; read Player 1 Door Y position
        LD      B,A
        RET

; *** stub if Left Picked ***
L23A3:  DEC     C               ; move Left a Block
        LD      A,10000011B     ; $83 for Wall Attributes
        JR      L2360                   ; (-$48)

; ****** Check Moves available Build Maze sub-routine [with BC=X/Y being checked] ***
; [returns D=Directions (bits 0=Up, 1=Down, 2=Left, 3=Right), E=number of directions]
L23A8:  LD      D,$00
        LD      A,B
        OR      A
        JR      Z,L23B9                 ; (+$0b) if at Top edge
        DEC     B
        RST     $08             ; User ReStarT #1 to find Walls location
        INC     B
        LD      A,(HL)          ; read Block above
        RRCA
        JR      C,L23B9                 ; (+$04) if Up already visited
        LD      A,D
        OR      $01
        LD      D,A
L23B9:  LD      A,(L4F4A)       ; read Maze Height
        DEC     A
        CP      B
        JR      Z,L23CB                 ; (+$0b) if at Top
        INC     B
        RST     $08             ; User ReStarT #1 to find Walls location
        DEC     B
        LD      A,(HL)
        RRCA
        JR      C,L23CB                 ; (+$04) if Down already visited
        LD      A,D
        OR      $02
        LD      D,A
L23CB:  LD      A,C
        OR      A
        JR      Z,L23DA                 ; (+$0b) if at Left edge
        DEC     C
        RST     $08             ; User ReStarT #1 to find Walls location
        INC     C
        LD      A,(HL)
        RRCA
        JR      C,L23DA                 ; (+$04) if Left already visited
        LD      A,D
        OR      $04
        LD      D,A
L23DA:  LD      A,(L4F4B)       ; read Maze Width
        DEC     A
        CP      C
        JR      NZ,L23E9                ; (+$08) if Not at Right
        LD      A,(L4F35)       ; read Player 2 Door Found [if at right edge]
        OR      A
        JR      Z,L23F0                 ; (+$09) to fake a right if Not found
        JR      L23F4                   ; (+$0b) else don't

L23E9:  INC     C
        RST     $08             ; User ReStarT #1 to find Walls location
        DEC     C
        LD      A,(HL)
        RRCA
        JR      C,L23F4                 ; (+$04) if Right already visited
L23F0:  LD      A,D
        OR      $08
        LD      D,A
; *** Count number of Direction bits set in D (into E)
L23F4:  LD      E,$00
        LD      A,D
        OR      A
L23F8:  RRA
        JR      NC,L23FC                ; (+$01)
        INC     E
L23FC:  OR      A
        JR      NZ,L23F8                ; (-$07)
        RET

; ****** Display "GET READY" String routine ***
L2400:  SYSSUK  STRDIS          ; UPI STRing DISplay
        DB      $32             ; ... X = 50
        DB      $1E             ; ... Y = 30
        DB      $2C             ; ... Options for X-OR Green on Yellow
        DW      L241E           ; ... String Address = 9246
        ;
        RET

; $$$$$$ start of score (Voice Mask = $0C for Tone C only) $$$
L2408:  MASTER  $20
        VOLUME  $FF,$0F         ; set A, B and C to full volume
        PUSHN   $03
; $$$ play three times through
L240E:  NOTE1   $08,$54         ; Staccato "Da-Dah-Dah-Da-Daah"
        NOTE1   $10,$54
        NOTE1   $10,$54
        NOTE1   $08,$70
        NOTE1   $12,$64
        DSJNZ   L240E
; end with "Dahhhh"
        NOTE1   $0C,$7E
        QUIET

; @@@@@@ "GET READY" text string @@@
L241E:  DB      'G','E','T',$20,'R','E','A','D','Y'
        DB      $00

; ****** display Alternate Font #0='X' and #1='O' [for Tic-Tac-Toe only] ***
L2428:  LD      IX,L27CD        ; point to Alt Font Descriptor
        SYSSUK  CHRDIS          ; UPI CHaRacter DISplay
        DB      $08             ; ... X = 8
        DB      $05             ; ... Y = 5
        DB      $68             ; ... Options for 2x X-OR Red on Yellow
        DB      $80             ; ... Character = 128 for Alt. Font 'X'
        ;
        SYSSUK  CHRDIS          ; UPI CHaRacter DISplay
        DB      $8F             ; ... X = 143
        DB      $05             ; ... Y = 5
        DB      $64             ; ... Options for 2x X-OR Blue on Yellow
        DB      $81             ; ... Character = 129 for Alt. Font 'O'
        ;
; ++++++ RST $10 / ReSTart #2 to Display Scores [for both] +++
L2438:  PUSH    DE
        SYSSUK  DISNUM          ; UPI DISplay NUMber - Player One Score
        DB      $07             ; ... X = 7
        DB      $16             ; ... Y = 22
        DB      $2C             ; ... Options for X-OR Green on Yellow
        DB      $02             ; ... Extended = 2 digits to display
        DW      L4F48           ; ... Number Address = 20296
        ;
        SYSSUK  DISNUM          ; UPI DISplay NUMber - Player Two Score
        DB      $8F             ; ... X = 143
        DB      $16             ; ... Y = 22
        DB      $2C             ; ... Options for X-OR Green on Yellow
        DB      $02             ; ... Extended = 2 digits to display
        DW      L4F49           ; ... Number Address = 20297
        ;
; [also entry #0 defaults table (not used)]
L2449:  POP     DE
        RET

; ====== Default settings by Difficulty ===
        DW      L245B           ; for entry #1 table
        DW      L2456           ; for entry #2 table
        DW      L2451           ; for entry #3 table

; === Defaults table #3 = Hard ===
L2451:  DB      $15             ; maze Height = 21
        DB      $1C             ; maze Width = 28
        DB      $04             ; X and Y Quantum = 4
        DB      $88             ; playfield Width = 136
        DB      $4D             ; playfield Height = 77

; === Defaults table #2 = Medium ===
L2456:  DB      $11             ; maze Height = 17
        DB      $17             ; maze Width = 23
        DB      $05             ; X and Y Quantum = 5
        DB      $8B             ; playfield Width = 139
        DB      $50             ; playfield Height = 80

; === Defaults table #1 = Easy ===
L245B:  DB      $0E             ; maze Height = 14
        DB      $13             ; maze Width = 19
        DB      $06             ; X and Y Quantum = 6
        DB      $8A             ; playfield Width = 138
        DB      $4D             ; playfield Height = 77

; @@@@@@ [misplaced] "TIC-TAC-TOE" text string @@@
L2460:  DB      'T','I','C','-','T','A','C','-','T','O','E'
        DB      $00

; ====== Color table #1 [for Both games] ===
L246C:  DB      $A3             ; color 3 Left = Green
        DB      $62             ; color 2 Left = Red
        DB      $FA             ; color 1 Left = Blue
; [also entry #0 offset (not used)]
L246F:  DB      $87             ; color 0 Left = Yellow ...

; ====== Table entry Offsets by Difficulty table ===
        DB      $08             ; entry #1 = +8 offset for "Easy"
        DB      $04             ; entry #2 = +4 offset for "Medium"
        DB      $00             ; entry #3 = +0 offset for "Hard"

; ====== Block patterns by (4 * Difficulty) + (0 to 3) table ===
; [note: each block can wall off only Right and/or Down movement]
L2473:  DW      L248B           ; entry #0 for Hard "Not OK"
        DW      L2490           ; entry #1 for ... "OK Right"
        DW      L2495           ; entry #2 for ... "OK Down"
        DW      L249B           ; entry #3 for ... "OK Right&Down"
        DW      L24A1           ; entry #4 for Medium "Not OK"
        DW      L24A7           ; entry #5 for ... "OK Right"
        DW      L24AD           ; entry #6 for ... "OK Down"
        DW      L24B4           ; entry #7 for ... "OK Right&Down"
        DW      L24BB           ; entry #8 for Easy "Not OK"
        DW      L24C2           ; entry #9 for ... "OK Right"
        DW      L24C9           ; entry #10 for ... "OK Down"
        DW      L24D1           ; entry #11 for ... "OK Right&Down"

; [note: Path X-OR back to Yellow patterns (NOT walls, they stay Green)]
; === Block #0 for Hard "Not OK" ===
L248B:  DB      $01             ; Width in Bytes
        DB      $03             ; Height in Rows
        DB      $E0             ; X X X . . . . .
        DB      $E0             ; X X X . . . . .
        DB      $E0             ; X X X . . . . .

; === Block #1 for Hard "OK Right" ===
L2490:  DB      $01             ; Width in Bytes
        DB      $03             ; Height in Rows
        DB      $F0             ; X X X X . . . .
        DB      $F0             ; X X X X . . . .
        DB      $F0             ; X X X X . . . .

; === Block #2 for Hard "OK Down" ===
L2495:  DB      $01             ; Width in Bytes
        DB      $04             ; Height in Rows
        DB      $E0             ; X X X . . . . .
        DB      $E0             ; X X X . . . . .
        DB      $E0             ; X X X . . . . .
        DB      $E0             ; X X X . . . . .

; === Block #3 for Hard "OK Right&Down" ===
L249B:  DB      $01             ; Width in Bytes
        DB      $04             ; Height in Rows
        DB      $F0             ; X X X X . . . .
        DB      $F0             ; X X X X . . . .
        DB      $F0             ; X X X X . . . .
        DB      $E0             ; X X X . . . . .

; === Block #4 for Medium "Not OK" ===
L24A1:  DB      $01             ; Width in Bytes
        DB      $04             ; Height in Rows
        DB      $F0             ; X X X X . . . .
        DB      $F0             ; X X X X . . . .
        DB      $F0             ; X X X X . . . .
        DB      $F0             ; X X X X . . . .

; === Block #5 for Medium "OK Right" ===
L24A7:  DB      $01             ; Width in Bytes
        DB      $04             ; Height in Rows
        DB      $F8             ; X X X X X . . .
        DB      $F8             ; X X X X X . . .
        DB      $F8             ; X X X X X . . .
        DB      $F8             ; X X X X X . . .

; === Block #6 for Medium "OK Down" ===
L24AD:  DB      $01             ; Width in Bytes
        DB      $05             ; Height in Rows
        DB      $F0             ; X X X X . . . .
        DB      $F0             ; X X X X . . . .
        DB      $F0             ; X X X X . . . .
        DB      $F0             ; X X X X . . . .
        DB      $F0             ; X X X X . . . .

; === Block #7 for Medium "OK Right&Down" ===
L24B4:  DB      $01             ; Width in Bytes
        DB      $05             ; Height in Rows
        DB      $F8             ; X X X X X . . .
        DB      $F8             ; X X X X X . . .
        DB      $F8             ; X X X X X . . .
        DB      $F8             ; X X X X X . . .
        DB      $F0             ; X X X X . . . .

; === Block #8 for Easy "Not OK" ===
L24BB:  DB      $01             ; Width in Bytes
        DB      $05             ; Height in Rows
        DB      $F8             ; X X X X X . . .
        DB      $F8             ; X X X X X . . .
        DB      $F8             ; X X X X X . . .
        DB      $F8             ; X X X X X . . .
        DB      $F8             ; X X X X X . . .

; === Block #9 for Easy "OK Right ===
L24C2:  DB      $01             ; Width in Bytes
        DB      $05             ; Height in Rows
        DB      $FC             ; X X X X X X . .
        DB      $FC             ; X X X X X X . .
        DB      $FC             ; X X X X X X . .
        DB      $FC             ; X X X X X X . .
        DB      $FC             ; X X X X X X . .

; === Block #10 for Easy "OK Down" ===
L24C9:  DB      $01             ; Width in Bytes
        DB      $06             ; Height in Rows
        DB      $F8             ; X X X X X . . .
        DB      $F8             ; X X X X X . . .
        DB      $F8             ; X X X X X . . .
        DB      $F8             ; X X X X X . . .
        DB      $F8             ; X X X X X . . .
        DB      $F8             ; X X X X X . . .

; === Block #11 for Easy "OK Right&Down" ===
L24D1:  DB      $01             ; Width in Bytes
        DB      $06             ; Height in Rows
        DB      $FC             ; X X X X X X . .
        DB      $FC             ; X X X X X X . .
        DB      $FC             ; X X X X X X . .
        DB      $FC             ; X X X X X X . .
        DB      $FC             ; X X X X X X . .
        DB      $F8             ; X X X X X . . .
; end of Path block monochrome patterns

; ====== Red color pattern for Player 1 block ===
L24D9:  DB      $01             ; Width in Bytes
        DB      $03             ; Height in Rows
        DB      $A8             ; 2 2 2 .
        DB      $A8             ; 2 2 2 .
        DB      $A8             ; 2 2 2 .

; ====== Blue color pattern for Player 2 block ===
L24DE:  DB      $01             ; Width in Bytes
        DB      $03             ; Height in Rows
        DB      $54             ; 1 1 1 .
        DB      $54             ; 1 1 1 .
        DB      $54             ; 1 1 1 .

; @@@@@@ "EASY" text string @@@
L24E3:  DB      'E','A','S','Y'
        DB      $00

; @@@@@@ "MEDIUM" text string @@@
L24E8:  DB      'M','E','D','I','U','M'
        DB      $00

; @@@@@@ "HARD" text string @@@
L24EF:  DB      'H','A','R','D'
; [also first write port soundblock]
L24F3:  DB      $00             ; set VOLume of Noise = 0

; ====== Sound Block initialize ===
        DB      $FF             ; set VOLumes A & B full
        DB      $00             ; set VOLume of C = 0
        DB      $00
        DB      $00
        DB      $00             ; set TONE B = 0 ??
        DB      $20             ; set TONE A = 32
        DB      $00

; === First DOIT table RCALLs [for AM joysticks & keys] ===
L24FB:  DB      $55             ; doit $15 for SJ0
        DW      L2532

        DB      $57             ; doit $17 for SJ1
        DW      L2536           ; else fall through to ...

; === Second DOIT table RCALL [for AM keypad only] ===
L2501:  DB      $53             ; doit $13 for SKYD
        DW      L2197

        DB      $C0             ; end of both AM tables

; @@@@@@ "AMAZING MAZE" text string @@@
L2505:  DB      'A','M','A','Z','I','N','G',$20,'M','A','Z','E'
        DB      $00

; ****** read Current Direction Nibble routine (and point to next) ***
L2512:  LD      HL,L4F53        ; point at Current path Nibble tracker
        LD      A,(HL)
        XOR     $01             ; toggle it [for next time]
        LD      (HL),A
        LD      HL,(L4F54)      ; read Current Direction byte
        LD      A,(HL)
        JR      NZ,L2525                ; (+$06) if tracker was 0
        RRA                     ; else shift high nibble down
        RRA
        RRA
        RRA
        JR      L2529                   ; (+$04)

L2525:  INC     HL              ; to next byte [next time]
        LD      (L4F54),HL
L2529:  LD      D,$01           ; set to Player Two
        LD      H,$00
        LD      L,H             ; zero HL
        AND     $0F
        LD      B,A             ; Fake a Joystick move
        RET

; ****** on sentry Joystick One handler [for AM only] ***
L2532:  LD      D,$00
        JR      L253D                   ; (+$07)

; ****** on sentry Joystick Two handler [for AM only] ***
L2536:  LD      D,$01
        LD      A,(NUMPLY)      ; read NUMber of PLaYers
        CP      D               ; only One player ?
        RET     Z               ; ignore second Joystick
; *** either handler continues to Apply Joystick (by D) Delta
L253D:  XOR     A
        LD      H,A             ; zero HL
        LD      L,A
        INC     A               ; speed = 1
        CALL    L2564           ; do Joystick handler on B
        LD      A,D             ; get Joystick being handled
        OR      A
        JR      Z,L254C                 ; (+$04) on Joystick 1
        LD      (L4F46),HL      ; write Player 2 Delta XY directions
        RET

L254C:  LD      (L4F3E),HL      ; write Player 1 Delta XY directions
        RET

; *** move Up subroutine [for Amazin Maze only] ***
L2550:  PUSH    AF
        NEG                     ; - XY Quantum [or -1]
        JR      L2556                   ; (+$01)

; *** move Down subroutine [for Amazin Maze only] ***
L2555:  PUSH    AF
L2556:  ADD     A,H             ; Y +/- Quantum [or 1]
        LD      H,A
        POP     AF
        RET

; *** move Left subroutine [for Amazin Maze only] ***
L255A:  PUSH    AF
        NEG                     ; - XY Quantum [or -1]
        JR      L2560                   ; (+$01)

; *** move Right subroutine [for Amazin Maze only] ***
L255F:  PUSH    AF
L2560:  ADD     A,L             ; X +/- Quantum [or 1]
        LD      L,A
        POP     AF
        RET

; ****** Joystick handler for all four directions ***
L2564:  BIT     0,B             ; test B for Up
        CALL    NZ,L2550
        BIT     1,B             ; test B for Down
        CALL    NZ,L2555
        BIT     2,B             ; test B for Left
        CALL    NZ,L255A
        BIT     3,B             ; test B for Right
        CALL    NZ,L255F
        RET

; *** more Interrupt sub-routine for X, only if P2 Delay is even ***
L2579:  LD      A,(L4F3E)       ; read Player 1 Delta X direction
        LD      HL,(L4F3A)      ; read Player 1 old XY locations
        ADD     A,L
        LD      L,A
        LD      (L4F3C),HL      ; write Player 1 new XY locations
        LD      A,(L4F46)       ; read Player 2 Delta X direction
        LD      HL,(L4F40)      ; read Player 2 old XY locations
        ADD     A,L
        LD      L,A
        JR      L25A1                   ; (+$13) to Move Blocks

; *** more Interrupt sub-routine for Y, only if P2 Delay is odd ***
L258E:  LD      A,(L4F3F)       ; read Player 1 Delta Y direction
        LD      HL,(L4F3A)      ; read Player 1 old XY locations
        ADD     A,H
        LD      H,A
        LD      (L4F3C),HL      ; write Player 1 new XY locations
        LD      A,(L4F47)       ; read Player 2 Delta Y direction
        LD      HL,(L4F40)      ; read Player 2 old XY locations
        ADD     A,H
        LD      H,A
; *** either continues to Move Both Blocks
L25A1:  LD      (L4F44),HL      ; write Player 2 new XY locations
        RST     $18             ; User ReStarT #3 to Erase Both Boxes
        IN      A,(INTST)       ; input INTerrupt STatus [to clear]
        CALL    L25D8           ; do Draw Red Box at New location
        IN      A,(INTST)       ; input INTerrupt STatus
        OR      A
        JR      NZ,L25B5                ; (+$06) to cancel move
        LD      HL,(L4F3C)      ; copy Player 1 new XY locations ...
        LD      (L4F3A),HL      ; to Player 1 old XY locations
L25B5:  CALL    L25D8           ; do Undraw Red Box at New location
        IN      A,(INTST)       ; input INTerrupt STatus [to clear]
        CALL    L25E1           ; do Draw Blue Box at New location
        IN      A,(INTST)       ; input INTerrupt STatus
        OR      A
        JR      NZ,L25C8                ; (+$06) to cancel move
        LD      HL,(L4F44)      ; copy Player 2 new XY locations ...
        LD      (L4F40),HL      ; to Player 2 old XY locations
L25C8:  CALL    L25E1           ; do Undraw Blue Box at New location
; ++++++ RST $18 / ReSTart #3 Erase or Draw BOTH Old Boxes +++
L25CB:  LD      DE,(L4F3A)      ; read Player 1 old XY locations
        CALL    L25DC           ; do Draw Red Box
        LD      DE,(L4F40)      ; read Player 2 old XY locations
        JR      L25E5                   ; (+$0d) to Draw Blue Box

; *** Draw New location RED Box routine
L25D8:  LD      DE,(L4F3C)      ; read Player 1 new XY locations
; *** Draw RED Box sub-routine
L25DC:  LD      HL,L24D9        ; point to Red color block
        JR      L25E8                   ; (+$07)

; *** Draw New location New BLUE Box routine
L25E1:  LD      DE,(L4F44)      ; read Player 2 new XY locations
; *** Draw BLUE Box sub-routine
L25E5:  LD      HL,L24DE        ; point to Blue color block
L25E8:  LD      A,$20           ; for X-OR in magic register
        SYSTEM  WRITP           ; UPI WRITe with Pattern size lookup
        ;
        RET

; end of Amazing Maze game code ***
;

; ********* Tic-Tac-Toe code starts here
; [note: never does an EI so Interrupt is never called] ***
;
; Squares: _1_|_2_|_3_
;          _4_|_5_|_6_
;           7 | 8 | 9  ...since Boxes are shapes. [Well, so are Squares, but Not here!]
;
L25ED:  CALL    L21D0           ; do Set-up Number of Players and Max Score
        DI                      ; +++ Disable Interrupts
        SYSSUK  FILL            ; UPI FILL memory with data
        DW      L4FB3           ; ... Memory Address = 20403
        DW      $0002           ; ... Byte Count = 2
        DB      $00             ; ... Data = 0 to set Turn and Went First to Player 1
        ;
        CALL    L21B3           ; do setup Interrupt Vector [why ??] and Clear Screen
; *** loop back for Next Game
L25FB:  LD      SP,L4FA0        ; [same location as in DogPatch ??]
        CALL    L21C8           ; do Clear Screen again
        SYSSUK  FILL            ; UPI FILL memory with data
        DW      L4FA0           ; ... Memory Address = 20384
        DW      $0013           ; ... Byte Count = 19 [to L4FB2]
        DB      $00             ; ... Data = 0 to clear Tic-Tac-Toe variables
        ;
        LD      A,$C0           ; output VERtical BLanking line 96 * 2 = 192
        OUT     (VERBL),A
        SYSTEM  INTPC           ; UPI INTerPret with Context create
        ;
        DO      FILL            ; UPI FILL memory with data
        DW      L4FB5           ; ... Memory Address = 20405
        DW      $0002           ; ... Byte Count = 2
        DB      $05             ; ... Data = 5 to preset both Squares [Center]
        ;
        DO      RECTAN          ; UPI paint a RECTANgle for Top line [of '#']
        DB      $1C             ; ... X = 28
        DB      $1E             ; ... Y = 30
        DB      $6C             ; ... Width = 108
        DB      $02             ; ... Height = 2
        DB      $FF             ; ... Color = Green
        ;
        DO      RECTAN          ; UPI paint a RECTANgle for Bottom line
        DB      $1C             ; ... X = 28
        DB      $40             ; ... Y = 64
        DB      $6C             ; ... Width = 108
        DB      $02             ; ... Height = 2
        DB      $FF             ; ... Color = Green
        ;
        DO      RECTAN          ; UPI paint a RECTANgle for Left line
        DB      $3E             ; ... X = 62
        DB      $00             ; ... Y = 0
        DB      $02             ; ... Width = 2
        DB      $60             ; ... Height = 96
        DB      $FF             ; ... Color = Green
        ;
        DO      RECTAN          ; UPI paint a RECTANgle for Right line
        DB      $64             ; ... X = 100
        DB      $00             ; ... Y = 0
        DB      $02             ; ... Width = 2
        DB      $60             ; ... Height = 96
        DB      $FF             ; ... Color = Green
        ;
        DONT    XINTC           ; UPI eXit INTerpreter with Context
        ;
        CALL    L2428           ; do Display X and O then fall into Show Scores
        RST     $20             ; User ReStarT #4 to Clear In-Game flag [not used ??]
        RST     $28             ; User ReStarT #5 to Draw Selection Box frame
; *** back here for SENSE transition #3 loop
L2632:  SYSSUK  SENTRY          ; UPI SENse TRansition
        DW      ALKEYS          ; ... Keymask Address for All KEYS
        ;
        SYSSUK  DOIT            ; UPI DOIT table #3, branch to translation handler
        DW      L263F           ; ... Table Address for Joysticks, Triggers and Keypad
        ;
        CALL    L2761           ; do Computer Player movement [if active]
        JR      L2632                   ; (-$0d) loop to SENSE3

; ====== Third DOIT table RCALLs [for TTT only] ===
L263F:  DB      $55             ; doit $15 for SJ0
        DW      L265F

        DB      $57             ; doit $17 for SJ1
        DW      L264F

        DB      $54             ; doit $14 for ST0
        DW      L26D4

        DB      $56             ; doit $16 for ST1
        DW      L26C7

        DB      $53             ; doit $13 for SKYD
        DW      L2197

        DB      $C0             ; end of TTT DoIt table

; ****** on sentry Joystick Two handler [for Tic-Tac-Toe only] ***
L264F:  CALL    L26A5           ; do Debounce Joystick 2
        LD      A,(NUMPLY)      ; read NUMber of PLaYers
        CP      $01             ; only One player ?
        RET     Z               ; ignore second joystick
        RST     $30             ; User ReStarT #6 to get Who's Turn [0/1]
        RET     Z               ; bail if Player One
        LD      HL,L4FB6        ; point to Player 2 Square [1 to 9]
        JR      L2667                   ; (+$08)

; ****** on sentry Joystick One handler [for Tic-Tac-Toe only] ***
L265F:  CALL    L26A5           ; do Debounce Joystick 1
        RST     $30             ; User ReStarT #6 to get Who's Turn [0/1]
        RET     NZ              ; bail if Player Two
        LD      HL,L4FB5        ; point to Player 1 Square [1 to 9]
; *** continue either Joystick handler here
L2667:  RST     $28             ; User ReStarT #5 to Erase Selector box
        LD      A,(HL)
        BIT     0,B
        CALL    NZ,L26BF        ; do Move on Up
        BIT     1,B
        CALL    NZ,L26B7        ; do Move on Down
        BIT     2,B
        CALL    NZ,L26B4        ; do Move on Left
        BIT     3,B
        CALL    NZ,L26B1        ; do Move on Right
        LD      (HL),A
; ++++++ RST $28 / ReSTart #5 to Draw (or Erase) Selection Box frame +++
L267E:  PUSH    BC
        PUSH    HL
        CALL    L2199           ; do Silence all sounds
        RST     $30             ; User ReStarT #6 to get Who's Turn [0/1]
        LD      HL,L4FB5        ; point to Player 1 Square [1 to 9]
        LD      C,$A8           ; for 4x X-OR in Red on Yellow
        JR      Z,L268E                 ; (+$03) skip on Player 1
        INC     HL              ; point to Player 2 Square [1 to 9]
        LD      C,$A4           ; for 4x X-OR in Blue on Yellow
L268E:  LD      A,(HL)          ; read Player Square [1 to 9]
        SYSSUK  INDEXW          ; UPI INDEX Word by A (for location)
        DW      L27B9           ; ... Base Address = 10169
        ;
        OUT     (VIBRA),A       ; output Square [1 to 9] to VIBRAto
        LD      A,$81           ; for Alt. Font 'O' character
        LD      IX,L27CD        ; point to Alt Font Descriptor
        SYSTEM  CHRDIS          ; UPI CHaRacter DISplay
        ;
        SYSSUK  PAWS            ; UPI PAUSE for 0.1 seconds (or so)
        DB      $06             ; ... Interrupts = 6
        ;
        SYSTEM  EMUSIC          ; UPI End playing MUSIC
        ;
        POP     HL
        POP     BC
        RET

; ****** debounce Joystick sub-routine [New movements Only] ***
L26A5:  LD      A,(L4FA0)       ; read Old Directions
        LD      C,B             ; save Joystick Direction
        XOR     B               ; note any changes
        AND     B               ; mask for leading edge only
        LD      B,A
        LD      A,C             ; unsave Joystick Direction
        LD      (L4FA0),A       ; write as New Directions
        RET

; *** Move on Right [for Tic-Tac-Toe only] ***
L26B1:  INC     A               ; add 1 to Square
        JR      L26B9                   ; (+$05) check wrap

; *** Move on Left [for Tic-Tac-Toe only] ***
L26B4:  DEC     A               ; subtract 1 from Square
        JR      L26C1                   ; (+$0a) check wrap

; *** Move on Down [for Tic-Tac-Toe only] ***
L26B7:  ADD     A,$03           ; add 3 to Square
L26B9:  CP      $0A
        RET     M               ; if still less than 10
        ADD     A,$F7           ; else subtract 9
        RET

; *** Move on Up [for Tic-Tac-Toe only] ***
L26BF:  ADD     A,$FD           ; subtract 3 from Square
L26C1:  CP      $01
        RET     P               ; if still more than 1
        ADD     A,$09           ; else add 9
        RET

; ****** on sentry Trigger Two handler [for Tic-Tac-Toe only] ***
L26C7:  BIT     4,B             ; debounce Trigger 2
        RET     Z
        RST     $30             ; User ReStarT #6 to get Who's Turn [0/1]
        RET     Z               ; ignore if Player One playing
; *** test a square for Player 2 AI [does "fail" if already filled in]
L26CC:  LD      HL,L4FB6        ; point to Player 2 Square [1 to 9]
        LD      BC,$6481        ; for Alt. Font 'O' 2x X-OR in Blue on Yellow
        JR      L26DF                   ; (+$0b)

; ****** on sentry Trigger One handler [for Tic-Tac-Toe only] ***
L26D4:  BIT     4,B             ; debounce Trigger 1
        RET     Z
        RST     $30             ; User ReStarT #6 to get Who's Turn [0/1]
        RET     NZ              ; ignore if Player Two playing
        LD      HL,L4FB5        ; point to Player 1 Square [1 to 9]
        LD      BC,$6880        ; for Alt. Font 'X' 2x X-OR in Red on Yellow
; *** continue on either Trigger handler here
L26DF:  PUSH    HL              ; store Player Square
        CALL    L2199           ; do Silence all sounds [clicks ??]
        LD      A,(HL)
        SYSSUK  INDEXB          ; UPI INDEX Byte by A [1 to 9]
        DW      L4FA0           ; ... Base Address = 20384
        ;
        OR      A
        JR      NZ,L26F6                ; (+$0b) if already filled in
        PUSH    AF
        RST     $30             ; User ReStarT #6 to get Who's Turn [0/1]
        JR      Z,L26F3                 ; (+$04)
        LD      (HL),$01        ; write a 1 if Player Two placed it
        JR      L26F5                   ; (+$02)

L26F3:  LD      (HL),$04        ; write a 4 if Player One placed it
L26F5:  POP     AF
L26F6:  POP     HL              ; restore Player Square
        RET     NZ              ; "fail" if already filled in
        LD      A,(HL)          ; read Player Square [1 to 9]
        OUT     (VIBRA),A       ; output Square to VIBRAto
        SYSSUK  INDEXW          ; UPI INDEX Word by A (for location)
        DW      L27B9           ; ... Base Address = 10169
        ;
        LD      HL,$0808        ; center 2x character in Square
        ADD     HL,DE
        EX      DE,HL
        LD      A,C             ; set Character and Options
        LD      C,B
        LD      IX,L27CD        ; point to Alt Font Descriptor
        SYSTEM  CHRDIS          ; UPI CHaRacter DISplay
        ;
        RST     $28             ; User ReStarT #5 to Erase Box frame
        LD      HL,L4FB3        ; point at Player Turn [0 or 1]
        LD      A,(HL)
        NEG                     ; get Two's Complement + 1
        INC     A
        LD      (HL),A          ; write Turn toggled [1 or 0]
; count the filled Squares in all Eight Lines
        LD      BC,$0301        ; for 3x "Left-to-Right" lines
        LD      DE,L4FAA        ; point to First Line to check
        LD      HL,L4FA1        ; point to Top Left square
L271E:  CALL    L2275           ; do Add Filled Squares [mis-placed]
        INC     HL              ; wrap to next row
        DJNZ    L271E                   ; (-$06) loop two more
        LD      BC,$0303        ; for 3x "Top-to-Bottom" lines
        LD      HL,L4FA1        ; point to Top Left square
L272A:  CALL    L2275           ; do Add Filled Squares [mis-placed]
        DEC     HL              ; back to top of next column
        DEC     HL
        DEC     HL
        DEC     HL
        DEC     HL
        DJNZ    L272A                   ; (-$0a) loop twice more
        LD      C,$04           ; Quantum for "Top Left to Bottom Right"
        LD      HL,L4FA1        ; point to Top Left square again
        CALL    L2275           ; do Add Filled Squares [mis-placed]
        LD      C,$02           ; Quantum for "Top Right to Bottom Left"
        LD      HL,L4FA1 + 2    ; point to Top Right square this time
        CALL    L2275           ; do Add Filled Squares [mis-placed]
; all 8 possible Lines done
        LD      DE,L4F48        ; point at Player One Score
        LD      A,$03           ; for Player 2 has "Three-in-a-Row"
        CALL    L2794           ; do Look for a 3 in Line Results
        JR      Z,L279D                 ; (+$4f) if Player Two won
        LD      A,$0C           ; for Player 1 has "Three-in-a-Row"
        CALL    L2794           ; do Look for a 12 in Line Results
        JR      Z,L279E                 ; (+$49) if Player One won
; check for finished a "tied" game
        LD      HL,L4FB2        ; point at Squares Filled counter
        LD      A,$08           ; test for stalemate condition
        CP      (HL)
        JR      Z,L27A4                 ; (+$47) if Last Square filled
; else next player's turn
        RST     $28             ; User ReStarT #5 to Draw other Selector
        INC     (HL)            ; Squares Filled + 1
        XOR     A               ; set Zero flag
        RET

; ****** move Computer Player (after DOIT table #3) sub-routine ***
L2761:  LD      A,(NUMPLY)      ; read NUMber of PLaYers
        CP      $01
        RET     NZ              ; bail if Player Two is Human
        RST     $30             ; User ReStarT #6 to get Who's Turn [0/1]
        RET     Z               ; bail if still Player One's turn
; *** Player Two computer AI starts
        LD      A,$02           ; for Player 2 has "Two-in-a-Row"
        CALL    L2794           ; do Look for A in Line Results
        JR      Z,L277B                 ; (+$0b) if 2 found
        LD      A,$08           ; for Player 1 has "Two-in-a-Row"
        CALL    L2794           ; do Look for A in Line Results
        JR      Z,L277B                 ; (+$04) if 8 found
        SYSSUK  RANGED          ; UPI RANGED random number
        DB      $06             ; ... Cutoff = 6 [see note on table]
        ;
        LD      C,A             ; pick a random number [0 to 5], or...
; need to place 'O' to Win or Block [with C=7-(Line Result found)]
L277B:  LD      A,C
        ADD     A,A             ; times 2
        ADD     A,C             ; plus 1
        SYSSUK  INDEXB          ; UPI INDEX Byte by A
        DW      L27E2           ; ... Base Address = 10210
        ;
; test First, Second, etc... Square in that Line [or the Next...]
L2782:  SYSSUK  PAWS            ; UPI PAUSE for about 0.62 seconds
        DB      $25             ; ... Interrupts = 37
        ;
        RST     $28             ; User ReStarT #5 to Erase Selector Box
        LD      A,(HL)          ; one of Three Squares in a Line
        INC     HL
        PUSH    HL              ; store Next Square location
        LD      (L4FB6),A       ; write Player 2 Square [1 to 9]
        RST     $28             ; User ReStarT #5 to Draw Selector Box
        CALL    L26CC           ; do Fake a sentry Trigger Two squeeze
        POP     HL              ; restore Next Square location
        JR      NZ,L2782                ; (-$11) if Square already filled
        RET

; *** look for A in Line Results sub-routine ***
L2794:  LD      BC,$0008
        LD      HL,L4FAA        ; point to first Line Result
        CPIR                    ; [set Z flag if found]
        RET

; ****** on Player Two won, increments Score ***
L279D:  INC     DE              ; point at Player Two Score
; *** on Player One won, increments Score here
L279E:  RST     $10             ; User ReStarT #2 to Erase old Scores
        LD      A,(DE)          ; winning Player Score + 1
        INC     A
        DAA
        LD      (DE),A
        RST     $10             ; User ReStarT #2 to Re-display Scores
; *** finish a Tie Game here
L27A4:  LD      HL,L4FB4        ; point at Went First [0/1]
        LD      A,(HL)
        XOR     $01             ; and Toggle it [not used at all ??]
        LD      (HL),A
        INC     HL              ; point at P1 Square [1 to 9]
        LD      (HL),A
        CALL    L21E7           ; do Play Music
        EX      DE,HL
        XOR     A               ; clear Joystick 1 tracking byte
        LD      (OSW0),A        ; [should be clearing L4FA0 ??]
        CALL    L2251           ; do check for Last Game, else...
        DB      $C3             ; [note: does a JP  L25FB here]
; [also entry #0 Offsets table (not used)]
L27B9:  DW      L25FB

; ====== digit YX Word offsets table ===
        DW      $001C           ; entry #1 for upper left
        DW      $0042           ; entry #2 for upper center
        DW      $0068           ; entry #3 for upper right
        DW      $221C           ; entry #4 for middle left
        DW      $2242           ; entry #5 for middle center
        DW      $2268           ; entry #6 for middle right
        DW      $441C           ; entry #7 for bottom left
        DW      $4442           ; entry #8 for bottom center
        DW      $4468           ; entry #9 for bottom right

; ========= Alternate Font Descriptor block [for Tic-Tac-Toe only] ===
L27CD:  DB      $80             ; FonT BASE [should be $00 ??]
        DB      $08             ; ... Frame Size X
        DB      $08             ; ... Frame Size Y
        DB      $01             ; ... x size in BYTEs
        DB      $07             ; ... Y SIZe in rows
        DW      L27D4           ; ... Pattern Table

; ====== Alternate Font table ===
; === character $80 = "X"
L27D4:  DB      $82             ; X . . . . . X .
        DB      $44             ; . X . . . X . .
        DB      $28             ; . . X . X . . .
        DB      $10             ; . . . X . . . .
        DB      $28             ; . . X . X . . .
        DB      $44             ; . X . . . X . .
        DB      $82             ; X . . . . . X .

; === character $81 = square
        DB      $FF             ; X X X X X X X X
        DB      $81             ; X . . . . . . X
        DB      $81             ; X . . . . . . X
        DB      $81             ; X . . . . . . X
        DB      $81             ; X . . . . . . X
        DB      $81             ; X . . . . . . X
        DB      $FF             ; X X X X X X X X

; ====== Three Squares that make a Line table ===
L27E2:  DB      $05             ; Line = 0 for '/' Diagonal
        DB      $03
        DB      $07
        DB      $05             ; Line = 1 for '\' Diagonal
        DB      $01
        DB      $09
        DB      $03             ; Line = 2 for Right Column
        DB      $06
        DB      $09
        DB      $02             ; Line = 3 for Center Column
        DB      $05
        DB      $08
        DB      $01             ; Line = 4 for Left Column
        DB      $04
        DB      $07
        DB      $08             ; Line = 5 for Bottom Row [note: ...
        DB      $07             ; ... Random is limited to this line
        DB      $09             ; ... so All squares will be checked]
        DB      $04             ; Line = 6 for Middle Row
        DB      $05
        DB      $06
        DB      $01             ; Line = 7 for Top Row
        DB      $02
        DB      $03

; === 6 bytes filler NOPs ===
        DB      $00
        DB      $00
        DB      $00
        DB      $00
        DB      $00
        DB      $00

; end of Tic-Tac-Toe game code ***
